program Compil32;

{
  Inno Setup
  Copyright (C) 1998-2000 Jordan Russell
  Portions by Martijn Laan
  For conditions of distribution and use, see LICENSE.TXT.

  Compiler

  $Id: Compil32.dpr,v 1.5 2001/10/03 00:39:12 jr Exp $
}

uses
  XPTheme,
  Windows,
  SysUtils,
  Forms,
  MBCSUtil,
  CompForm in 'CompForm.pas' {CompileForm},
  CmnFunc in 'CmnFunc.pas',
  CmnFunc2 in 'CmnFunc2.pas',
  CompMsgs in 'CompMsgs.pas',
  CompInt in 'CompInt.pas',
  CompOptions in 'CompOptions.pas' {OptionsForm},
  CompStartup in 'CompStartup.pas' {StartupForm},
  CompWizard in 'CompWizard.pas' {WizardForm},
  CompWizardFile in 'CompWizardFile.pas' {WizardFileForm};

{$R *.RES}

procedure CreateMutexes;
{ Creates the two mutexes used by the Inno Setup's own installer/uninstaller to
  see if the compiler is still running.
  One of the mutexes is created in the global name space (which makes it
  possible to access the mutex across user sessions in Windows XP); the other
  is created in the session name space (because versions of Windows NT prior
  to 4.0 TSE don't have a global name space and don't support the 'Global\'
  prefix). }
const
  MutexName = 'InnoSetupCompilerAppMutex';
  SECURITY_DESCRIPTOR_REVISION = 1;  { Win32 constant not defined in Delphi 3 }
var
  SecurityDesc: TSecurityDescriptor;
  SecurityAttr: TSecurityAttributes;
begin
  { By default on Windows NT, created mutexes are accessible only by the user
    running the process. We need our mutexes to be accessible to all users, so
    that the mutex detection can work across user sessions in Windows XP. To
    do this we use a security descriptor with a null DACL. }
  InitializeSecurityDescriptor (@SecurityDesc, SECURITY_DESCRIPTOR_REVISION);
  SetSecurityDescriptorDacl (@SecurityDesc, True, nil, False);
  SecurityAttr.nLength := SizeOf(SecurityAttr);
  SecurityAttr.lpSecurityDescriptor := @SecurityDesc;
  SecurityAttr.bInheritHandle := False;
  CreateMutex (@SecurityAttr, False, MutexName);
  CreateMutex (@SecurityAttr, False, 'Global\' + MutexName);  { don't localize }
end;

procedure CheckParams;
  procedure Error;
  begin
    Application.MessageBox (SCompilerCommandLineHelp3, SCompilerFormCaption,
      MB_OK or MB_ICONEXCLAMATION);
    Halt (1);
  end;
var
  P, I: Integer;
  S: String;
  SkipParam: Boolean;
begin
  P := ParamCount;
  if P <> 0 then begin
    SkipParam := False;
    for I := 1 to P do begin
      if SkipParam then begin
        SkipParam := False;
        Continue;
      end;
      S := ParamStr(I);
      if CompareText(S, '/CC') = 0 then
        CommandLineCompile := True
      else if CompareText(S, '/WIZARD') = 0 then begin
        if I = P then
          Error;
        CommandLineWizard := True;
        CommandLineWizardName := ParamStr(I+1);
        SkipParam := True;
      end else if (S[1] = '/') or (CommandLineFilename <> '') then
        Error
      else
        CommandLineFilename := ExpandFileName(S);
    end;
  end;
  if (CommandLineCompile or CommandLineWizard) and (CommandLineFilename = '') then
    Error;
end;

begin
  if Win32Platform = VER_PLATFORM_WIN32s then begin
    Application.MessageBox (SCompilerNotOnWin32s, nil, MB_OK or MB_ICONSTOP);
    Exit;
  end;
  CreateMutexes;
  CheckParams;

  { The 'with' is so that the Delphi IDE doesn't mess with these }
  with Application do begin
    if CommandLineWizard then
      Title := CommandLineWizardName
    else
      Title := SCompilerFormCaption;
    HelpFile := ExtractFilePath(ParamStr(0)) + 'isetup.hlp';
  end;

  Application.CreateForm(TCompileForm, CompileForm);
  Application.Run;
end.
