program ISCC;
{$APPTYPE CONSOLE}

{
  Inno Setup
  Copyright (C) 1998-2001 Jordan Russell
  For conditions of distribution and use, see LICENSE.TXT.

  Command-line compiler

  $Id: ISCC.dpr,v 1.4 2001/03/27 23:44:27 jr Exp $
}

uses
  Windows, SysUtils, MBCSUtil, CmnFunc2, CompInt;

{$R *.RES}

var
  F: TextFile;
  CurLine: String;
  StartTime, EndTime: DWORD;
  WantAbort: Boolean;

function ConsoleCtrlHandler (dwCtrlType: DWORD): BOOL; stdcall;
begin
  { Abort gracefully when Ctrl+C/Break is pressed }
  WantAbort := True;
  Result := True;
end;

function CompilerCallbackProc (Code: Integer; var Data: TCompilerCallbackData;
  AppData: Longint): Integer; stdcall;
begin
  if WantAbort then begin
    Result := iscrRequestAbort;
    Exit;
  end;
  Result := iscrSuccess;
  case Code of
    iscbReadScript: begin
        if Data.Reset then
          Reset (F);
        if not Eof(F) then begin
          Readln (F, CurLine);
          Data.LineRead := PChar(CurLine);
        end;
      end;
    iscbNotifyStatus:
      Writeln (Data.StatusMsg);
    iscbNotifySuccess: begin
        EndTime := GetTickCount;
        Writeln;
        Writeln ('Successful compile (', (EndTime - StartTime) / 1000 : 0 : 3,
          ' sec). Resulting Setup program filename is:');
        Writeln (Data.OutputExeFilename);
      end;
    iscbNotifyError:
      if Assigned(Data.ErrorMsg) then begin
        Write ('Error');
        if Data.ErrorLine <> 0 then
          Write (' on line ', Data.ErrorLine);
        if Assigned(Data.ErrorFilename) then
          Write (' in ', Data.ErrorFilename);
        Writeln (': ', Data.ErrorMsg);
      end;
  end;
end;

procedure Go;
var
  ScriptFilename, ScriptPath: String;
  ExitCode: Integer;
  Ver: PCompilerVersionInfo;
  Params: TCompileScriptParams;
begin
  Writeln ('Inno Setup 2.0 Command-Line Compiler (a)');
  Writeln ('Copyright (C) 1998-2001 Jordan Russell. All rights reserved.');
  Writeln ('Portions by Martijn Laan');
  Writeln;
  if (ParamCount <> 1) or (ParamStr(1) = '/?') then begin
    Writeln ('Usage:  iscc scriptfile.iss');
    Halt (1);
  end;

  ScriptFilename := ExpandFilename(ParamStr(1));
  ScriptPath := ExtractFilePath(ScriptFilename);

  AssignFile (F, ScriptFilename);
  FileMode := fmOpenRead or fmShareDenyWrite;
  Reset (F);
  ExitCode := 0;
  try
    Ver := ISDllGetVersion;
    Writeln ('Compiler engine version: ', Ver.Title, ' ', Ver.Version);
    Writeln;

    FillChar (Params, SizeOf(Params), 0);
    Params.Size := SizeOf(Params);
    Params.SourcePath := PChar(ScriptPath);
    Params.CallbackProc := CompilerCallbackProc;

    StartTime := GetTickCount;
    case ISDllCompileScript(Params) of
      isceInvalidParam: begin
          ExitCode := 1;
          Writeln ('Internal error 1.');
        end;
      isceCompileFailure: begin
          ExitCode := 2;
          Writeln ('Compile aborted.');
        end;
    end;
  finally
    CloseFile (F);
  end;
  if ExitCode <> 0 then
    Halt (ExitCode);
end;

procedure ShowExceptionMsg (Msg: String);
begin
  if (Msg <> '') and (AnsiLastChar(Msg)^ > '.') then Msg := Msg + '.';
  Writeln (Msg);
end;

begin
  SetConsoleCtrlHandler (@ConsoleCtrlHandler, True);
  try
    Go;
  except
    on E: Exception do begin
      { Show a friendlier exception message. (By default, Delphi prints out
        the exception class and address.) }
      ShowExceptionMsg (E.Message);
      Halt (2);
    end;
  end;
end.
